import hashlib
import hmac
import logging

import django
from asana.error import InvalidRequestError
from django.conf import settings
if django.VERSION >= (2, 0, 0):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

logger = logging.getLogger(__name__)


def sign_sha256_hmac(secret, message):
    if type(message) != bytes:
        message = bytes(message.encode('utf-8'))
    if type(secret) != bytes:
        secret = bytes(secret.encode('utf-8'))
    return hmac.new(secret, message, digestmod=hashlib.sha256).hexdigest()


def set_webhook(client, project_id):
    target = '{}{}'.format(
        settings.DJASANA_WEBHOOK_URL,
        reverse('djasana_webhook', kwargs={'remote_id': project_id}))
    logger.debug('Setting webhook at %s', target)
    try:
        client.webhooks.create({
            'resource': project_id,
            'target': target,
        })
    except InvalidRequestError as error:
        logger.warning(error)
        logger.warning('Target url: %s', target)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-19 14:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0013_adds_on_delete'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(blank=True, max_length=254, null=True, verbose_name='email address'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-30 09:02
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0011_modifies_choices'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='djasana.Team', to_field='remote_id'),
        ),
    ]



from django.conf.urls import url, include
from djasana.settings import settings

from .views import WebhookView

urlpatterns = [
    url(settings.DJASANA_WEBHOOK_PATTERN or r'^', include([
        url(r'^project/(?P<remote_id>\d+)/$', view=WebhookView.as_view(), name='djasana_webhook'),
    ]))]



# Generated by Django 2.0.5 on 2018-06-05 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0002_adds_project_start_on'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='is_edited',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='story',
            name='is_pinned',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.0.5 on 2018-06-05 09:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0001_squashed_0015_adds_task_start_on'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='start_on',
            field=models.DateField(blank=True, null=True),
        ),
    ]



from django import forms
from django.contrib import admin
from django.contrib.admin import widgets
from django.utils.safestring import mark_safe

from djasana import models


def asana_link(obj):
    return mark_safe('<a href="{}" target="_blank">View on Asana</a>'.format(obj.asana_url()))


class ParentRawIdWidget(widgets.ForeignKeyRawIdWidget):

    def url_parameters(self):
        params = super().url_parameters()
        object_ = self.attrs.get('object', None)
        if object_:
            # Filter parent choices by project
            params['projects__id__exact'] = object_.projects.first().pk
        return params


@admin.register(models.Attachment)
class AttachmentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'name', 'parent', asana_link)
    raw_id_fields = ('parent',)
    readonly_fields = (asana_link,)


@admin.register(models.Project)
class ProjectAdmin(admin.ModelAdmin):
    date_hierarchy = 'created_at'
    list_display = ('__str__', 'owner', 'archived', asana_link)
    list_filter = ('workspace', 'team', 'archived')
    readonly_fields = ('workspace', 'team', asana_link)


class TaskForm(forms.ModelForm):

    class Meta:
        fields = ('name', 'assignee', 'completed', 'completed_at',
                  'due_at', 'due_on', 'parent', 'notes', 'projects')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['parent'].widget = ParentRawIdWidget(
                rel=self.instance._meta.get_field('parent').remote_field,
                admin_site=admin.site,
                # Pass the object to attrs
                attrs={'object': self.instance}
            )


@admin.register(models.Task)
class TaskAdmin(admin.ModelAdmin):
    date_hierarchy = 'created_at'
    form = TaskForm
    list_display = ('name', 'assignee', 'completed', 'due', asana_link)
    list_filter = ('completed', 'projects__workspace', 'projects__team', 'assignee', 'projects')
    raw_id_fields = ('assignee', 'parent')
    readonly_fields = (asana_link,)
    search_fields = ('remote_id', 'name')


@admin.register(models.Team)
class TeamAdmin(admin.ModelAdmin):
    list_display = ('__str__', asana_link)
    readonly_fields = (asana_link,)


@admin.register(models.User)
class UserAdmin(admin.ModelAdmin):
    list_display = ('__str__',)
    readonly_fields = (asana_link,)


@admin.register(models.Workspace)
class WorkspaceAdmin(admin.ModelAdmin):
    list_display = ('__str__', asana_link)
    readonly_fields = (asana_link,)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-06 15:35
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0012_project_team_allows_null'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='owner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='projects_owned', to='djasana.User', to_field='remote_id'),
        ),
        migrations.AlterField(
            model_name='project',
            name='team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djasana.Team', to_field='remote_id'),
        ),
        migrations.AlterField(
            model_name='task',
            name='assignee',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='assigned_tasks', to='djasana.User', to_field='remote_id'),
        ),
        migrations.AlterField(
            model_name='task',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djasana.Task', to_field='remote_id'),
        ),
    ]
